﻿using System.Diagnostics;
using System.Text;
using System.Windows;
using Microsoft.Azure.Devices;

namespace GearController
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        static ServiceClient serviceClient;
        static string DeviceId = "W10IoT"; //Device Id obtained from Device Explorer
        static string cs = "HostName=VasuW10IoT.azure-devices.net;SharedAccessKeyName=iothubowner;SharedAccessKey=xxxxxxxx"; //Primary Key
        //static string eventHubName = "iothub-ehub-vasuw10iot-xxxxxxxxx"; //obtained from IoT Cloud-> All Settings/Messaging/Event Hub-compatible name - only used if you have storage space
        static bool Connection = false;
        static int prevSpeed;
        static bool Auto = false;
        static bool MotorOn = false;
        private const float INITIALSPEED = 40;

        /* ********
         * 
         * Message will be formatted for Start / Stop; Gear Shift; Manual / Auto; Speed;
         * Formats as follow:
         * Start message will always have two additional parameter - speed and Gear Mode separated by :
         * "Start:Manual" -> Start button pressed with Manual mode
         * "Start:Auto" -> Start button pressed with Auto mode.
         * "Stop" -> Stop Button pressed.
         * "GearShift" -> Gear shift Pressed
         * "Manual" -> Manual button Pressed
         * "Auto" -> Button Pressed
         * "Speed:050" -> Accelerator value is 50
         *  
         * ****/

        public MainWindow()
        {
            InitializeComponent();
            ConnectToIoTHub();
            prevSpeed = (int)slAccelerator.Value;
        }

        private void ConnectToIoTHub()
        {
            if (Connection) //already connected
                return;
            serviceClient = ServiceClient.CreateFromConnectionString(cs);
            if (serviceClient == null)
                tbMessage.Text = "Unable to connect to IoT hub";
            else
            {
                tbMessage.Text = "Ready to send message to IoT hub";
                Connection = true;
            }
            
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void slAccelerator_Change(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (slAccelerator != null && serviceClient!= null)
            {
                slAccelerator.Value = (int)slAccelerator.Value;
                int speed = (int)slAccelerator.Value;
                string msg = string.Format("Speed:{0:D3}",speed);
                if (prevSpeed == speed)
                    return;
                SendToDeviceCloudMessage(msg);
                tbMessage.Text += "\nMessage Sent -> " + msg;
                tbMessage.ScrollToEnd();
                prevSpeed = speed;
            }

        }

        private void SendToDeviceCloudMessage(string msg)
        {
            if (serviceClient == null || !Connection)
                return;
            var commandMessage = new Message(Encoding.ASCII.GetBytes(msg));
            Stopwatch sw = new Stopwatch();
            sw.Start();
            while (sw.ElapsedMilliseconds < 5000) //5 seconds for timeout
            {
                serviceClient.SendAsync(DeviceId, commandMessage);
                break;
            }
            //tbMessage.Text += string.Format("\nTime spent for sending in milliseconds: {0}",sw.ElapsedMilliseconds);
            sw.Stop();
            if (sw.ElapsedMilliseconds > 5000)
            {
                tbMessage.Text = "IoT Hub is not connected. Reconnect";
                Connection = false;
            }
            tbMessage.ScrollToEnd();

        }

        private void btnConnect_Click(object sender, RoutedEventArgs e)
        {
            ConnectToIoTHub();
        }

        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            if (serviceClient != null)
            {
                slAccelerator.Value = INITIALSPEED;
                string msg;
                if (Auto)
                    msg = string.Format("Start:Auto");
                else
                    msg = string.Format("Start:Manual");
                SendToDeviceCloudMessage(msg);
                tbMessage.Text += "\nMessage Sent -> " + msg;
                MotorOn = true;

            }
            else
                tbMessage.Text += "\nEither not connected to IoT or Engine already running";
            tbMessage.ScrollToEnd();


        }

        private void btnStop_Click(object sender, RoutedEventArgs e)
        {
            if (serviceClient != null && MotorOn)
            {
                string msg;
                msg = "Stop";
                SendToDeviceCloudMessage(msg);
                tbMessage.Text += "\nMessage Sent -> " + msg;
                MotorOn = false;
            }
            else
                tbMessage.Text += "\nEither not connected to IoT or Engine not running";
            tbMessage.ScrollToEnd();

        }

        private void btnGearShift_Click(object sender, RoutedEventArgs e)
        {
            if (Auto)
                return;
            if (serviceClient != null && MotorOn)
            {
                string msg;
                msg = "GearShift";
                SendToDeviceCloudMessage(msg);
                tbMessage.Text += "\nMessage Sent -> " + msg;
            }
            else
                tbMessage.Text += "\nEither not connected to IoT or Engine not running";
            tbMessage.ScrollToEnd();

        }

        private void rbAuto_Click(object sender, RoutedEventArgs e)
        {
            if (Auto)  //Already on cruise control. nothing to do
                return;
            slAccelerator.Value = INITIALSPEED;
            rbManual.IsChecked = false;
            Auto = true;
            if (serviceClient != null)
            {
                string msg;
                msg = "Auto";
                SendToDeviceCloudMessage(msg);
                tbMessage.Text += "\nMessage Sent -> " + msg;
            }
            else
                tbMessage.Text += "\nNot connected to IoT";
            tbMessage.ScrollToEnd();
        }

        private void rb_Manual_Click(object sender, RoutedEventArgs e)
        {
            if (!Auto)  //Already on Manual control. nothing to do
                return;
            slAccelerator.Value = INITIALSPEED;

            rbAuto.IsChecked = false;
            Auto = false;
            if (serviceClient != null)
            {
                string msg;
                msg = "Manual";
                SendToDeviceCloudMessage(msg);
                tbMessage.Text += "\nMessage Sent -> " + msg;
            }
            else
                tbMessage.Text += "\nNot connected to IoT";
            tbMessage.ScrollToEnd();

        }
    }
}
